#ifndef CATALINA_PLUGIN__H
#define CATALINA_PLUGIN__H
/*
 * The compiler expects _sys_plugin to look like this ... DO NOT ALTER!!!
 */
extern long _sys_plugin(long code, long param);

/*
 * Plugin types:
 */
#define VMM 0
#define HMI 1
#define LIB 2
#define FLA 3
#define FLB 4
#define RTC 5
#define FIL 6
#define SIO 7
#define DUM 8
#define CGI 9
#define KBD 10
#define SCR 11
#define MOU 12
#define PRX 13
#define GAM 14
#define SND 15
#define ADC 16

/*
 * Plugin requests:
 */
extern int _short_plugin_request (long plugin_type, long code, long param);
extern int _long_plugin_request (long plugin_type, long code, long param);
extern int _long_plugin_request_2 (long plugin_type, long code, long par1, long par2);
extern float _float_request(long plugin_type, long code, float a, float b);
/*
 * strcuture of a request block (used to communicate with a plugin):
 */
typedef struct {
   unsigned int request;
   unsigned int response;
} request_t;

/*
 * Structure of the registry - the registry is basically a list of 8 pointers 
 * to request blocks (one for each cog).
 *
 * NOTE: to use the pointers, the upper byte must be masked off since this 
 *       contains the type of the plugin registered. Macros are provided
 *       to use these values - REGISTRY_ENTRY, REGISTERED_TYPE() and 
 *       REQUEST_BLOCK() (see below). Here is an example of their use:
 *
 *           int type;
 *           request_t *rqst;
 *
 *           type = REGISTERED_TYPE(i);
 *           rqst = REQUEST_BLOCK(i);
 */
typedef unsigned registry_t[8];

/*
 * Registry functions:
 */
extern unsigned _registry();
extern void _register_plugin(int cog_id, int plugin_type);
extern void _unregister_plugin(int cog_id);
extern int  _locate_plugin(int plugin_type);

/*
 * macros to simplify access to the registry. In each case, i is the cog.
 */
#define REGISTRY_ENTRY(i) ((*((registry_t *)(_registry())))[i])
#define REGISTERED_TYPE(i) (REGISTRY_ENTRY(i)>>24)
#define REQUEST_BLOCK(i) ((request_t *) (REGISTRY_ENTRY(i) & 0xFFFF))

#endif
