#include "catalina_threads.h"
#include "catalina_cog.h"

/*
 * Include the dynamic kernel formatted as a C array. The array was produced
 * using the following commands:
 *
 *       homespun Catalina_LMM_threaded_dynamic.spin -b -o LMM
 *       spinc LMM.binary > LMM_threaded_array.h
 */
#include "LMM_threaded_array.h"


/*
 * _thread_cog : start a C function as a thread in a new multi-threaded cog.
 *
 * On entry:
 *    func  : address of C function to run (defined as 'int f(int, char **)')
 *    stack : address of TOP of stack to use (i.e. points just after last long)
 * On exit:
 *    cog   : cog used, or -1 on any error
 */
int _thread_cog(_thread func, unsigned long *stack, int argc, char *argv[]) {
   int cog;
   struct {
      unsigned long REG;
      unsigned long PC;
      unsigned long SP;
      unsigned long argc;
      unsigned long argv;
      unsigned long exit;
   } cog_data;
 
   cog_data.REG = _registry(); // registry address
   cog_data.PC  = (unsigned long)func;  // address of C function
   cog_data.SP  = (unsigned long)stack; // top of stack
   cog_data.argc = (unsigned long)argc; // argc value
   cog_data.argv = (unsigned long)argv; // argv value
   cog_data.exit = (unsigned long)NULL; // exit processing
   cog = _coginit((int)&cog_data>>2, (int)LMM_threaded_array>>2, ANY_COG);
   _thread_wait(100); // small delay for cog to initialize
   return cog;
}

