#ifndef CATALINA_COG__H
#define CATALINA_COG__H

/*
 * clock control constants and functions
 */
#define RCFAST 0x00
#define RCSLOW 0x01
#define XINPUT 0x02
#define XTAL_1 0x2A
#define XTAL_2 0x32
#define XTAL_3 0x3A
#define PLL1X  0x41
#define PLL2X  0x42
#define PLL4X  0x43
#define PLL8X  0x44
#define PLL16X 0x45

extern unsigned _clockfreq();  // return current frequency - e.g. 80000000
extern unsigned _clockmode();  // return current mode - e.g. XTAL_1 + PLL16X
extern _clockinit(unsigned mode, unsigned freq); // set mode and freq


/*
 * Cog control constants and functions
 */
#define ANY_COG 0x8 /* may be used in _coginit instead of a cog number */

extern int _cogid();

extern int _coginit(int par, int addr, int cogid);

extern int _cogstop(int cogid);

extern int _coginit_C(void func(void), unsigned long *stack);

extern int _coginit_Spin(void *code, void *data, void *stack, int start, int offs);

/*
 * define some macros useful for retrieving Spin long or word values from 
 * a byte array, specifying the array name and the byte offset in the array:
 */
#define SPIN_LONG(var, off) (*((long *)(&var[off])))
#define SPIN_WORD(var, off) (*((short *)(&var[off])))

/*
 * Lock management constants and functions
 */
extern int _locknew();

extern int _lockclr(int lockid);

extern int _lockret(int lockid);

extern int _lockset(int lockid);

/*
 * Wait constants and functions
 * NOTE: _waitpeqb and _waitpneb are not implemented on the current propeller
 */

#define INA 0 /* use in _waitpeq, _waitpne */
#define INB 1 /* use in _waitpeq, _waitpne */

extern int _waitcnt(unsigned count);

extern int _waitvid(unsigned colors, unsigned pixels);

extern int _waitpeq(unsigned mask, unsigned result, int a_or_b);

extern int _waitpne(unsigned mask, unsigned result, int a_or_b);

/*
 * Register access functions
 * NOTE: _inb, _dirb and _outb are not implemented on the current propeller
 */
extern unsigned _cnt();

extern unsigned _ina();
extern unsigned _inb();

extern unsigned _get_dira();
extern unsigned _get_dirb();

extern unsigned _dira(unsigned mask, unsigned direction);
extern unsigned _dirb(unsigned mask, unsigned direction);

extern unsigned _outa(unsigned mask, unsigned output);
extern unsigned _outb(unsigned mask, unsigned output);

extern unsigned _ctra(unsigned mask, unsigned control);
extern unsigned _ctrb(unsigned mask, unsigned control);

extern unsigned _frqa(unsigned mask, unsigned frequency);
extern unsigned _frqb(unsigned mask, unsigned frequency);

extern unsigned _phsa(unsigned mask, unsigned phase);
extern unsigned _phsb(unsigned mask, unsigned phase);

extern unsigned _vcfg(unsigned mask, unsigned config);
extern unsigned _vscl(unsigned mask, unsigned scale);

/*
 * counter control constants
 */
#define COUNTER_OFF 0x00

/* video mode */
#define COUNTER_PLL_INTERNAL 0x01

#define COUNTER_PLL  0x02
#define COUNTER_NCO  0x04
#define COUNTER_DUTY 0x06

/* bit-wise OR one of these to COUNTER_{PLL,NCO,DUTY} */
#define COUNTER_SINGLE_ENDED 0x00
#define COUNTER_DIFFERENTIAL 0x01

#define COUNTER_POS      0x08
#define COUNTER_POS_EDGE 0x0A
#define COUNTER_NEG      0x0C
#define COUNTER_NEG_EDGE 0x0E

/* bit-wise OR one of these to COUNTER_{POS,NEG}{,_EDGE} */
#define COUNTER_NO_FEEDBACK 0x00
#define COUNTER_FEEDBACK    0x01

#define COUNTER_LOGIC_NEVER         0x10
#define COUNTER_LOGIC_NOTA_AND_NOTB 0x11
#define COUNTER_LOGIC_A_AND_NOTB    0x12
#define COUNTER_LOGIC_NOTB          0x13
#define COUNTER_LOGIC_NOTA_AND_B    0x14
#define COUNTER_LOGIC_NOTA          0x15
#define COUNTER_LOGIC_A_NEQ_B       0x16
#define COUNTER_LOGIC_NOTA_OR_NOTB  0x17
#define COUNTER_LOGIC_A_AND_B       0x18
#define COUNTER_LOGIC_A_EQ_B        0x19
#define COUNTER_LOGIC_A             0x1A
#define COUNTER_LOGIC_A_OR_NOTB     0x1B
#define COUNTER_LOGIC_B             0x1C
#define COUNTER_LOGIC_NOTA_OR_B     0x1D
#define COUNTER_LOGIC_A_OR_B        0x1E
#define COUNTER_LOGIC_ALWAYS        0x1F

#define COUNTER_CONTROL_REGISTER(mode, plldiv, bpin, apin) (((mode) << 26) | ((plldiv) << 23) | ((bpin) << 9) | (apin))

#endif
